@@ -1,6 +1,6 @@
 /*
  * buffers.c - buffer management for the Linux file system degragmenter.
- * $Id: buffers.c,v 1.5 1997/08/17 14:32:47 linux Exp $
+ * $Id: buffers.c,v 1.6 1998/01/25 13:51:51 linux Exp $
  *
  * Copyright (C) 1992, 1993, 1997 Stephen Tweedie (sct@dcs.ed.ac.uk)
  * 
@@ -329,7 +329,7 @@ void read_current_block (Block nnr, char * addr)
 		return;
 	check_zone_nr(nnr);
 
-	offset = block_size * nnr;
+	offset = (loff_t) block_size * nnr;
 	
 	if (offset != nlseek (IN, offset, SEEK_SET))
 	{
@@ -360,7 +360,7 @@ void write_current_block (Block nnr, char * addr)
 			(long) nnr, (int) addr);
 	check_zone_nr(nnr);
 
-	offset = block_size * nnr;
+	offset = (loff_t) block_size * nnr;
 	
 	if (offset != nlseek (IN, offset, SEEK_SET))
 	{
