@@ -1,10 +1,12 @@
 /*
  * defrag.c - the Linux file system degragmenter.
- * $Id: defrag.c,v 1.6 1994/07/12 13:31:42 linux Exp $
+ * $Id: defrag.c,v 1.4 1997/08/17 14:23:57 linux Exp $
+ *
+ * changes are Copyleft  (C) 1997 Ulrich Habel (espero@b31.hadiko.de)
  *
  * changes are Copyleft  (C) 1997 Anthony Tong (antoine@eci.com)
  *
- * Copyright (C) 1992, 1993 Stephen Tweedie (sct@dcs.ed.ac.uk)
+ * Copyright (C) 1992, 1993, 1997 Stephen Tweedie (sct@dcs.ed.ac.uk)
  *
  * Copyright (C) 1992 Remy Card (card@masi.ibp.fr) 
  *
@@ -34,7 +36,7 @@
 int debug = 0;
 #endif
 
-char * RCSID = "$Id: defrag.c,v 0.61 1997/01/28 13:29:09 linux Exp $";
+char * RCSID = "$Id: defrag.c,v 1.4 1997/08/17 14:23:57 linux Exp $";
 
 int die_on_io_error = 1;
 int io_errors = 0;
@@ -45,12 +47,6 @@ int verbose = 0;
 int show = 0;
 int show_version = 0; 
 
-#ifdef EXTFS       /* The use of EXT fs is deprecated */
-   int voyer_mode = 0;
-#else
-   int voyer_mode = 1;
-#endif
-
 int test_disk = 0;
 int salvage = 0;
 int no_bad_inode = 0;
@@ -67,6 +63,12 @@ unsigned int zones = 0, block_size = 0;
 struct d_inode inode_buffer;
 int current_inode = 0;
 char super_block_buffer[BLOCK_SIZE];
+#ifdef EXT2FS
+  struct ext2_super_block s;
+#endif
+#ifdef MINIX
+  struct minix_super_block s;
+#endif
 unsigned char * inode_map = NULL;
 Block *inode_average_map = NULL;
 char *inode_priority_map = NULL;
@@ -394,11 +396,11 @@ void walk_inode (struct d_inode *inode, enum walk_zone_mode mode)
 	int i;
 	
 	for (i = 0; i < DIRECT_ZONES ; i++)
-		walk_zone (i + inode->i_zone, mode);
-	walk_zone_ind (DIRECT_ZONES + inode->i_zone, mode);
-	walk_zone_dind ((DIRECT_ZONES+1) + inode->i_zone, mode);
+	    walk_zone  ((Block *) ( i                 + inode->i_zone), mode);
+	walk_zone_ind  ((Block *) ( DIRECT_ZONES      + inode->i_zone), mode);
+	walk_zone_dind ((Block *) ((DIRECT_ZONES + 1) + inode->i_zone), mode);
 #ifdef HAS_TIND
-	walk_zone_tind ((DIRECT_ZONES+2) + inode->i_zone, mode);
+	walk_zone_tind ((Block *) ((DIRECT_ZONES + 2) + inode->i_zone), mode);
 #endif
 }
 
@@ -418,11 +420,10 @@ void read_fixed_zones (void)
     {
 	/* Reserved blocks don't count as bad blocks */
 	int tmp = badblocks;
-	
 	int i;
+
 	for (i=1; i < FIRST_USER_INODE; i++) {        
-	    if ((i == EXT2_ROOT_INO) ||  /* Allow optimization of root
-inode */
+	    if ((i == EXT2_ROOT_INO) ||  /* Allow optimization of root */
 		(i == bad_block_inode))  /* Done with them already */
 		continue;
 	    if (!inode_in_use (i))
@@ -665,7 +666,7 @@ void sort_inodes (void)
 	if (bad_block_inode)
 		inode_order_map[used_inodes++] = bad_block_inode;
 
-	for (i=FIRST_USER_INODE; i<=INODES; i++)
+	for (i = FIRST_USER_INODE; i <= INODES; i++)
 	{
 		if (inode_in_use(i))
 			inode_order_map[used_inodes++] = i;
@@ -779,8 +780,8 @@ int main (int argc, char ** argv)
 		bad_block_inode = user_bad_inode;
 	}
 	read_fixed_zones ();
-        show_super_stats();
 
+        show_super_stats();
         show_reserved_blocks();
 
 	next_block_to_fill = FIRSTZONE;
@@ -801,8 +802,9 @@ int main (int argc, char ** argv)
 #endif
         clear_attr(AT_SELECTED);
 	remap_disk_blocks ();
-        update_display();
 
+        update_display();
+ 
 	if (!readonly)
 	{
 		salvage_free_zones ();
@@ -812,24 +814,24 @@ int main (int argc, char ** argv)
 	if (show || voyer_mode)
 	{  char s[256];
                 clear_comments();
-		sprintf (s,"%d buffer reads in %d group%s",
-			count_buffer_reads, count_read_groups,
-			count_read_groups==1 ? "" : "s");
+		sprintf (s,"%6d buffer  reads in %3d group%s",
+			 count_buffer_reads, count_read_groups,
+			 count_read_groups == 1 ? "" : "s");
                 add_comment(s);
-		sprintf (s,"  of which %d read-aheads.",
-			count_buffer_read_aheads);
+		sprintf (s,"    of which %7d  read-aheads.",
+			 count_buffer_read_aheads);
                 add_comment(s);
-		sprintf (s,"%d buffer writes in %d group%s",
-			count_buffer_writes, count_write_groups,
-			count_write_groups==1 ? "" : "s");
+		sprintf (s,"%6d buffer writes in %3d group%s",
+			 count_buffer_writes, count_write_groups,
+			 count_write_groups==1 ? "" : "s");
                 add_comment(s);
-		sprintf (s,"  %d migrations, %d forces.",
+		sprintf (s,"%6d migrations, %8d forces",
 			 count_buffer_migrates, count_buffer_forces);
                 add_comment(s);
                 display_comments(" Relocation statistics ");
-                        
 	}
-        if (voyer_mode) {
+        
+	if (voyer_mode) {
                 done_screen(TRUE);
         }        
 	if (io_errors)
