@@ -22,6 +22,7 @@
 #endif
 
 #include "types.h"
+#include <sys/param.h>
 
 #ifdef EXTFS
 #include "ext.h"
@@ -178,6 +179,11 @@ int seek_to_inode(int);
 
 #endif /* !MISC_C */
 
+#if defined(MISC_C) && !defined(__OPTIMIZE__)
+#define inline
+#endif
+
+#ifdef i386bogus
 #define bitop(name,op) \
 extern int name(char * addr,unsigned int nr) \
 { \
@@ -188,13 +194,23 @@ extern int name(char * addr,unsigned int nr) \
 	return __res; \
 }
 
-#if defined(MISC_C) && !defined(__OPTIMIZE__)
-#define inline
-#endif
-
 inline bitop(bit,"")
 inline bitop(setbit,"s")
 inline bitop(clrbit,"r")
+#else	/* ! i386 */
+#define bitop(name,cmd) \
+extern int name(char * addr,unsigned int nr) \
+{ \
+	return (int)cmd ;\
+}
+
+/*
+inline bitop(bit,"((FIELD)&(BIT))")
+inline bitop(setbit,"((FIELD)!=(BIT))")
+inline bitop(clrbit,"((FIELD)&=(~BIT))")
+*/
+#define bit(field,bit) isset(field,bit)
+#endif /* ! i386 */
 
 /* Now for the prototypes for misc.c */
 void check_mount(char *device_name);
@@ -222,3 +238,5 @@ extern void io_error(const char *message);
 loff_t defrag_llseek (unsigned int fd, loff_t offset,
 		      unsigned int origin);
 
+extern __off_t __llseek __P ((int __fd, __off_t __offset, int __whence));
+extern __off_t llseek __P ((int __fd, __off_t __offset, int __whence));
