@@ -1,4 +1,3 @@
-
 /* This is the result of my first experiments with the ext2 file system.
  * The code is rather messy and sometimes simply wrong,
  * but for me it's better than nothing.
@@ -10,11 +9,12 @@
  * Anthony Tong (antoine@eci.com)
  */
 
- /* Known bugs:
-  * 1. Picture option uses ncurses and requires a terminfo description
-  *    from the ncurses package. Default terminfo files (from SLS at least)
-  *    are broken.
-  */
+/* Known bugs:
+ * 1. Picture option uses ncurses and requires a terminfo description
+ *    from the ncurses package. Default terminfo files (from SLS at least)
+ *    are broken.
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -22,20 +22,16 @@
 #include <fcntl.h>
 #include <time.h>
 #include <linux/ext2_fs.h>
+#include "types.h"
+#include "ext2.h"
 #include "display.h"
 
-#define FIRST_USER_INODE ((s.s_rev_level == EXT2_GOOD_OLD_REV) ? \
-				EXT2_GOOD_OLD_FIRST_INO : \
-				s.s_first_ino)
-
 #ifndef TRUE
   #define FALSE 0 
   #define TRUE  1
 #endif  
 typedef unsigned char byte;
 
-int voyer_mode = 1;
-
 static int super_valid = FALSE;
 static int block_size = 0;
 static int groups = 0;  
@@ -43,6 +39,8 @@ static ulong *indblock;  /* indirection blocks used in file allocation scheme */
 static ulong *dindblock;
 static ulong *tindblock;
 
+loff_t defrag_llseek (unsigned int fd, loff_t offset,
+		      unsigned int origin);
 
 struct ext2_super_block s;
 struct group_table {
@@ -108,7 +106,8 @@ void load_groups(void) {
          die("Unable to read group descriptors\n");
          
       if ((s.s_blocks_per_group % 8) != 0) 
-         printf("Strange blocks_per_group (%lu) value",s.s_blocks_per_group);
+	  printf("Strange blocks_per_group (%lu) value", 
+		 (unsigned long) s.s_blocks_per_group);
       size = s.s_blocks_per_group / 8;
       gt[i].block_bmap = malloc_chk(size);
       lseek(IN,gt[i].gr.bg_block_bitmap*block_size,SEEK_SET);
@@ -116,7 +115,8 @@ void load_groups(void) {
          die("Unable to read block bitmap\n");
                
       if ((s.s_inodes_per_group % 8) != 0) 
-         printf("Strange inodes_per_group (%lu) value",s.s_blocks_per_group);
+         printf("Strange inodes_per_group (%lu) value",
+		(unsigned long) s.s_blocks_per_group);
       size = s.s_inodes_per_group / 8;
       gt[i].inode_bmap = malloc_chk(size);   
       lseek(IN,gt[i].gr.bg_inode_bitmap*block_size,SEEK_SET);
@@ -158,9 +158,11 @@ void load_inode(struct ext2_inode *n,ulong inode_no) {
  */
    if (bg != last_accessed_bg) {     /* Load a new group */
       ulong len = sizeof(struct ext2_inode) * s.s_inodes_per_group;
-      lseek(IN, block_size*bg->bg_inode_table,SEEK_SET);
-      if (read(IN,inode_group,len)!=len) 
-         exit(1);
+      loff_t offset = (loff_t) block_size * bg->bg_inode_table;
+      
+      if (defrag_llseek(IN, offset, SEEK_SET) != offset ||
+	  read(IN, inode_group, len) != len) 
+         die("Unable to read inode");
       last_accessed_bg = bg;   
    }
    memcpy(n,inode_group + (inode_no % s.s_inodes_per_group),
@@ -168,8 +170,10 @@ void load_inode(struct ext2_inode *n,ulong inode_no) {
 }
 
 void load_block(void * buf, ulong bn) {
-   lseek(IN, bn*block_size,SEEK_SET);
-   if (read(IN,buf,block_size)!=block_size) 
+   loff_t offset = (loff_t) bn * block_size;
+	
+   if (defrag_llseek(IN, offset, SEEK_SET) != offset ||
+       read(IN,buf,block_size)!=block_size) 
       die("Unable to read block");
 }
   
@@ -378,19 +382,30 @@ void check_allocation(struct ext2_inode *n, int quiet) {
 
 void dump_super(void) {
   printf("SUPERBLOCK:\n");
-  printf("Inodes count:%lu\n",s.s_inodes_count);	/* Inodes count */
-  printf("Blocks count:%lu\n",s.s_blocks_count);	/* Blocks count */
-  printf("Reserv blocks count:%lu\n",s.s_r_blocks_count);    /* Reserved blocks count */
-  printf("Free blocks   count:%lu\n",s.s_free_blocks_count); /* Free blocks count */
-  printf("Free inodes   count:%lu\n",s.s_free_inodes_count); /* Free inodes count */
-  printf("First data block   :%lu\n",s.s_first_data_block);  /* First Data Block */
-  printf("Block size   :%u\n",EXT2_MIN_BLOCK_SIZE << s.s_log_block_size); 
-  printf("Fragment size:%u\n",EXT2_MIN_FRAG_SIZE << s.s_log_frag_size); 
-  printf("Blocks per group:%lu\n",s.s_blocks_per_group);/* # Blocks per group */
-  printf("Frags  per group:%lu\n",s.s_frags_per_group); /* # Fragments per group */
-  printf("Inodes per group:%lu\n",s.s_inodes_per_group);/* # Inodes per group */
-  printf("mount time:%s", ctime(&s.s_mtime)); /* Mount time */
-  printf("write time:%s", ctime(&s.s_wtime)); /* Write time */
+  printf("Inodes count:%lu\n", 
+	 (unsigned long) s.s_inodes_count);      /* Inodes count */
+  printf("Blocks count:%lu\n", 
+	 (unsigned long) s.s_blocks_count);	 /* Blocks count */
+  printf("Reserv blocks count:%lu\n", 
+	 (unsigned long) s.s_r_blocks_count);    /* Reserved blocks count */
+  printf("Free blocks   count:%lu\n",
+	 (unsigned long) s.s_free_blocks_count); /* Free blocks count */
+  printf("Free inodes   count:%lu\n",
+	 (unsigned long) s.s_free_inodes_count); /* Free inodes count */
+  printf("First data block   :%lu\n",
+	 (unsigned long) s.s_first_data_block);  /* First Data Block */
+  printf("Block size   :%u\n", EXT2_MIN_BLOCK_SIZE << s.s_log_block_size); 
+  printf("Fragment size:%u\n", EXT2_MIN_FRAG_SIZE  << s.s_log_frag_size); 
+  printf("Blocks per group:%lu\n",
+	 (unsigned long) s.s_blocks_per_group);  /* # Blocks per group */
+  printf("Frags  per group:%lu\n",
+	 (unsigned long) s.s_frags_per_group);   /* # Fragments per group */
+  printf("Inodes per group:%lu\n",
+	 (unsigned long) s.s_inodes_per_group);  /* # Inodes per group */
+  printf("mount time:%s", 
+	 ctime ((time_t *) &s.s_mtime));    /* Mount time */
+  printf("write time:%s", 
+	 ctime ((time_t *) &s.s_wtime));    /* Write time */
   printf("magic:0x%X", s.s_magic); 		       /* Magic signature */
   if (s.s_magic == EXT2_SUPER_MAGIC) printf(" (OK)\n");
   else printf(" (???) ");
@@ -411,12 +426,12 @@ void dump_inode(ulong inode_no) {
    else if (S_ISLNK(n.i_mode)) printf("(link)\n");
    else if (S_ISFIFO(n.i_mode)) printf("(fifo)\n");
    else if (S_ISSOCK(n.i_mode)) printf("(socket)\n");
-   printf("Owner Uid %d ",(uint)n.i_uid);
+   printf("Owner Uid %d ",(uint) n.i_uid);
    printf("Group Id %d\n",(uint) n.i_gid);
-   printf("File size %lu\n",n.i_size);
-   printf("Access time      : %s",ctime(&n.i_atime));
-   printf("Creation time    : %s",ctime(&n.i_ctime));
-   printf("Modification time: %s",ctime(&n.i_mtime));
+   printf("File size %lu\n", (unsigned long) n.i_size);
+   printf("Access time      : %s", ctime ((time_t *) &n.i_atime));
+   printf("Creation time    : %s", ctime ((time_t *) &n.i_ctime));
+   printf("Modification time: %s", ctime ((time_t *) &n.i_mtime));
    if (inode_is_busy(inode_no)) {
       if (n.i_dtime!=0) printf("ERROR: bitmap is 1  ");
    }
@@ -424,19 +439,19 @@ void dump_inode(ulong inode_no) {
       if (n.i_dtime==0 && n.i_ctime!=0) printf("ERROR: bitmap is 0\n ");
       
    if (n.i_dtime!=0)
-      printf("Deletion time    : %s",ctime(&n.i_dtime));
+      printf("Deletion time    : %s", ctime ((time_t *) &n.i_dtime));
    printf("Links count: %d\n", (uint) n.i_links_count);
                              /* in 512 byte blocks for some unknown reason */
-   printf("512-Blocks count: %ld\n",n.i_blocks); 
-   if (n.i_flags !=0) printf("Flags 0x%lX\n",n.i_flags);
-   printf("Version: %lu\n",n.i_version);
+   printf("512-Blocks count: %lu\n", (unsigned long) n.i_blocks); 
+   if (n.i_flags !=0) printf ("Flags 0x%lX\n", (unsigned long) n.i_flags);
+   printf("Version: %lu\n", (unsigned long) n.i_version);
    
    if (n.i_file_acl!=0 || n.i_dir_acl!=0) {    
-      printf("File ACL: %lu  ",n.i_file_acl);
-      printf("Directory ACL: %lu\n",n.i_dir_acl);	
+      printf("File ACL: %lu  ", (unsigned long) n.i_file_acl);
+      printf("Directory ACL: %lu\n", (unsigned long) n.i_dir_acl);	
    }
    if (n.i_faddr!=0 || n.i_frag!=0 || n.i_fsize!=0) {     
-      printf("Fragment address: %lu\n",n.i_faddr);
+      printf("Fragment address: %lu\n", (unsigned long) n.i_faddr);
       printf("Fragment number: %u\n",n.i_frag);
       printf("Fragment size: %u\n",n.i_fsize);
    }
@@ -458,12 +473,15 @@ void dump_groups(void) {
    for (group_no = 0; group_no < groups; group_no++) {
       bg = &gt[group_no].gr;
       printf("GROUP %d\n",group_no);   
-      printf("Blocks bitmap location:%lu\n",bg->bg_block_bitmap);
-      printf("Inodes bitmap location:%lu\n",bg->bg_inode_bitmap);
-      printf("Inodes table location:%lu\n",bg->bg_inode_table);
-      printf("Free blocks in the group:%u\n",bg->bg_free_blocks_count);
-      printf("Free inodes in the group:%u\n",bg->bg_free_inodes_count);
-      printf("Directories count:%u\n",bg->bg_used_dirs_count);
+      printf("Blocks bitmap location:%lu\n", 
+	     (unsigned long) bg->bg_block_bitmap);
+      printf("Inodes bitmap location:%lu\n",
+	     (unsigned long) bg->bg_inode_bitmap);
+      printf("Inodes table location:%lu\n",
+	     (unsigned long) bg->bg_inode_table);
+      printf("Free blocks in the group:%u\n", bg->bg_free_blocks_count);
+      printf("Free inodes in the group:%u\n", bg->bg_free_inodes_count);
+      printf("Directories count:%u\n", bg->bg_used_dirs_count);
    }
 }
 
@@ -487,10 +505,13 @@ void dump_block_bitmap(void) {
 void dump_block(ulong bn) {
    int i;
    byte *buf;
+   loff_t offset;
    
    printf("Block %lu:\n",bn);
    buf = malloc_chk(block_size);
-   if (lseek(IN,bn*block_size,SEEK_SET)<0)
+
+   offset = (loff_t) bn * block_size;
+   if (defrag_llseek(IN, offset, SEEK_SET) != offset)
        die("Can't seek there\n");
    if (block_size!=read(IN,buf,block_size))
        die("Can't read the block\n");
@@ -515,7 +536,7 @@ void report_fragm(void) {
          avr_fr_size=0,
          avr_dist_file=0,
          avr_fr_size_file=0;
-         
+
    for (inode_no=FIRST_USER_INODE; inode_no <= s.s_inodes_count; inode_no++) {
       load_inode(&n,inode_no);
       if (n.i_dtime != 0 || n.i_ctime==0) { /* FIXME: is it a free inode? */
@@ -528,7 +549,7 @@ void report_fragm(void) {
       check_allocation(&n,TRUE);
       if (fragments > 1) {
          printf("Inode: %lu, frag: %lu, file size %lu, ",
-                inode_no, fragments, n.i_size);
+                inode_no, fragments, (unsigned long) n.i_size);
          total_frag_files++;       
          total_fragments += fragments;
       }
@@ -611,7 +632,7 @@ void report_group_fill(void) {
       for (inode_no=gr_no*s.s_inodes_per_group+1; 
            inode_no <= s.s_inodes_per_group*(gr_no+1); 
            inode_no++) {
-         if (inode_no < FIRST_USER_INODE && inode_no!=EXT2_ROOT_INO) 
+         if ((inode_no < FIRST_USER_INODE) && (inode_no != EXT2_ROOT_INO)) 
              continue;  
             
          load_inode(&n,inode_no);
@@ -651,14 +672,14 @@ void report_group_fill(void) {
       }   
       bg = &gt[gr_no].gr;
       printf("GROUP %ld:   ",gr_no);
-      printf("%ld used inodes\n",
-             s.s_inodes_per_group - bg->bg_free_inodes_count);
+      printf("%lu used inodes\n",
+             (unsigned long) (s.s_inodes_per_group - bg->bg_free_inodes_count));
       printf("%lu blocks required for all files, ",
              gr_blocks);
       printf("%lu blocks used, %u free\n",
              blocks_in_the_group - bg->bg_free_blocks_count,
              bg->bg_free_blocks_count);
-      printf("%ld files have %lu blocks outside the group,\n",frag_files,
+      printf("%ld files have %lu blocks outside the group,\n", frag_files,
               frag_blocks);
       printf("%ld files (%lu blocks) are located "
 	     "completely out of the group\n",
@@ -761,14 +782,14 @@ void show_map(void) {                /* make block map */
    if (total_dirs==0) total_dirs=1;
    
    
-   sprintf(str,"%ld%% files fragmented",(frag_files*100)/total_files);
+   sprintf(str,"  %3ld%% files fragmented",(frag_files*100)/total_files);
    add_comment(str);
-   sprintf(str,"%ld%% directiories fragmented",(frag_dirs*100)/total_dirs);
+   sprintf(str,"  %3ld%% directiories fragmented",(frag_dirs*100)/total_dirs);
    add_comment(str);
-   sprintf(str,"%ld%% files left their groups",
+   sprintf(str,"  %3ld%% files left their groups",
 	   (far_files*100)/(total_files+total_dirs));
-   add_comment(str);
-   display_comments(" Fragmentation ");
+   add_comment (str);
+   display_comments (" Fragmentation ");
 }
 
 enum commands {      
@@ -872,10 +893,12 @@ int main(int argc, char *argv[]) {
       case CM_B_DUMP: dump_block(StartBlock);                
                       break;
       case CM_DISPLAY:init_screen(s.s_blocks_count);
-                      show_map();
-                      display_legend(AT_REG|AT_DIR|AT_BITMAP|
-                                     AT_INODE|AT_GROUP|AT_SUPER|AT_BAD);
-                      done_screen(TRUE);
+	              show_map();
+		      display_legend(AT_REG   | AT_DIR|AT_BITMAP |
+                                     AT_INODE | AT_GROUP         | 
+				     AT_SUPER | AT_BAD           | 
+				     AT_FRAG);
+		      done_screen(TRUE);
                       /* Fall through */
       case CM_NONE:   break;
    }
