@@ -105,7 +105,7 @@ void read_tables (void)
      * (>2GB) partition.  
      */
     end_of_device = ((loff_t) ZONES) * BLOCK_SIZE - BLOCK_SIZE;
-    if (llseek(IN, end_of_device, SEEK_SET) != end_of_device)
+    if (defrag_llseek(IN, end_of_device, SEEK_SET) != end_of_device)
 	    die ("Error seeking to end of filesystem");
 	
     read_groups();
@@ -194,7 +194,7 @@ void show_super_stats(void) {
 void init_inode_bitmap (void)
 {
 	int i,size;
-        ulong pos;
+        loff_t pos;
 
 	if (debug)
 		printf ("DEBUG: init_inode_bitmap()\n");
@@ -207,9 +207,9 @@ void init_inode_bitmap (void)
 	memset (inode_map, 0, ((INODES + 1) / 8));
 
         for (i = 0; i < groups; i++) {
-                pos = bg[i].bg_inode_bitmap * block_size;
+                pos = (loff_t) bg[i].bg_inode_bitmap * block_size;
                 if (debug)
-                        printf("Group:%d inode_bitmap at:%lu\n",i,pos);
+                        printf("Group:%d inode_bitmap at:%llu\n",i,pos);
                 if (pos!=nlseek(IN,pos,SEEK_SET)) 
                         die("seek failed reading inode bitmap");
            
@@ -307,7 +307,8 @@ void init_zone_maps (void)
 {
 	Block i;
         int n;
-        ulong size,pos;
+        ulong size;
+	loff_t pos;
         
         size = Super.s_blocks_per_group >> 3;
                            /* The last group on the disk can be shorter, 
@@ -318,7 +319,7 @@ void init_zone_maps (void)
 		die ("Unable to allocate zone bitmap\n");
 
         for (n = 0; n < groups; n++) {
-                pos = bg[n].bg_block_bitmap * block_size;
+                pos = (loff_t) bg[n].bg_block_bitmap * block_size;
                 if (pos!=nlseek(IN,pos,SEEK_SET)) 
                         die("seek failed reading zone bitmap");
            
@@ -368,10 +369,11 @@ void write_tables (void)
 void salvage_free_zones (void) {
    Block blk;
    int n;
-   ulong size,pos;
+   ulong size;
    ulong bmp_zones;     /* Number of zones defined by bitmap size,  */
                         /* can be larger than the actual zone count */
-        
+   loff_t pos;
+   
    size = Super.s_blocks_per_group >> 3;
    bmp_zones = groups * Super.s_blocks_per_group;   
 
@@ -396,7 +398,7 @@ void salvage_free_zones (void) {
             bm_mark_zone(blk);
 
    for (n = 0; n < groups; n++) {
-           pos = bg[n].bg_block_bitmap * block_size;
+           pos = (loff_t) bg[n].bg_block_bitmap * block_size;
            if (pos!=nlseek(IN,pos,SEEK_SET)) 
                    die("seek failed writing zone bitmap");
       
@@ -418,7 +420,7 @@ int seek_to_inode(int i) {
         p = &bg[i / Super.s_inodes_per_group];
    
 	if (nlseek(IN,
-            p->bg_inode_table*block_size + sizeof(struct d_inode)* 
+            (loff_t) p->bg_inode_table*block_size + sizeof(struct d_inode)* 
             (i % Super.s_inodes_per_group), 
             SEEK_SET)<0) {
 		io_error ("Can't seek to inode table");
