@@ -2,7 +2,6 @@
  * ext2.h - extfs-specific include file for the Linux file system 
  * degragmenter. 
  *
- * changes are Copyleft  (C) 1997 Anthony Tong (antoine@eci.com)
  * Copyleft  (C) 1993 Alexey Vovenko (vovenko@ixwin.ihep.su)
  * Copyright (C) 1992, 1993 Stephen Tweedie (sct@dcs.ed.ac.uk)
  * Copyright (C) 1992 Remy Card (card@masi.ibp.fr)
@@ -15,11 +14,17 @@
 
 #include <linux/fs.h>
 #include <linux/ext2_fs.h>
+#include <linux/ext2_fs_i.h>
+#include <linux/ext2_fs_sb.h>
 
 #define HAS_TIND
 
 #define DIRECT_ZONES     EXT2_NDIR_BLOCKS
 
+#define ROOT_INO         EXT2_ROOT_INO
+#define BAD_INO          EXT2_BAD_INO
+#define FIRST_USER_INODE EXT2_FIRST_INO(&s)
+
 typedef unsigned long Block;
 
 #define d_inode ext2_inode
@@ -43,15 +48,6 @@ typedef unsigned long Block;
 #define NORM_FIRSTZONE	1                          
 #define INODES_PER_BLOCK (BLOCK_SIZE >> 2)
 
-#define ROOT_INO         EXT2_ROOT_INO
-#define BAD_INO          EXT2_BAD_INO
-
-/* Replacement for EXT2_FIRST_INO(s) */
-#define FIRST_USER_INODE ((Super.s_rev_level == EXT2_GOOD_OLD_REV) ? \
-				EXT2_GOOD_OLD_FIRST_INO : \
-				Super.s_first_ino)
-
-
     /* This structure is used to calculate, how many blocks are claimed 
      * by the inodes in the each group. 
      * native_blocks is set by update_group_population and counts blocks
@@ -85,7 +81,6 @@ struct groups_population {
    ulong last_block;
 };
 
-ulong choose_block(ulong inode);
-void check_group_population(void);
-void update_group_population(ulong znr,enum walk_zone_mode mode,ulong inode);
-   
+ulong choose_block (ulong inode);
+void check_group_population (void);
+void update_group_population (ulong znr, enum walk_zone_mode mode, ulong inode);
