@@ -1,7 +1,7 @@
 /*
  * minix.h - minix-specific include file for the Linux file system 
  * degragmenter. 
- * $Id: minix.h,v 1.1 1994/05/22 22:56:53 linux Exp $
+ * $Id: minix.h,v 1.3 1997/08/10 16:32:10 linux Exp $
  *
  * Copyright (C) 1992 Stephen Tweedie (sct@dcs.ed.ac.uk)
  *
@@ -16,6 +16,8 @@
 
 #include <linux/fs.h>
 #include <linux/minix_fs.h>
+#include <linux/minix_fs_i.h>
+#include <linux/minix_fs_sb.h>
 
 #define NAME_LEN MINIX_NAME_LEN
 
